﻿Module Module_Utils

    Friend GCI As Globalization.CultureInfo = Globalization.CultureInfo.InvariantCulture

    Friend RTBLines() As String

    ' ----------------------------------------------------------------------------------
    '  StartTime STOP WATCH
    ' ----------------------------------------------------------------------------------
    Friend StartTime As Stopwatch = New Stopwatch
    Friend Sub SetStartTime()
        StartTime.Reset()
        StartTime.Start()
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Swap
    ' ----------------------------------------------------------------------------------
    Friend Sub Swap(Of T)(ByRef d1 As T, ByRef d2 As T)
        Dim d As T = d2
        d2 = d1
        d1 = d
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" the complete word pointed by the "selectionIndex"
    ' ----------------------------------------------------------------------------------
    Friend Function GetCompleteWord(ByVal text As String, _
                                    ByVal selectionIndex As Int32) As String
        Dim start, length As Int32
        FindWordPointers(text, selectionIndex, start, length)
        Return text.Substring(start, length)
    End Function

    ' ----------------------------------------------------------------------------------------------
    '  Extract from "text" the complete word pointed by the "selectionIndex" (short words version)
    ' ----------------------------------------------------------------------------------------------
    Friend Function GetCompleteWord_Short(ByVal text As String, _
                                          ByVal selectionIndex As Int32) As String
        Dim start, length As Int32
        FindWordPointers_ShortWords(text, selectionIndex, start, length)
        Return text.Substring(start, length)
    End Function

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" start and length of the word pointed by the "selectionIndex"
    ' ----------------------------------------------------------------------------------
    Friend Sub FindWordPointers(ByVal text As String, _
                                ByVal selectionIndex As Int32, _
                                ByRef start As Int32, _
                                ByRef lenght As Int32)
        Const separators As String = " " + vbCr + vbLf + vbTab
        Dim i1, i2 As Int32
        For i1 = selectionIndex To 1 Step -1
            If InStr(separators, text(i1 - 1)) > 0 Then
                Exit For
            End If
        Next
        For i2 = selectionIndex To text.Length - 1
            If InStr(separators, text(i2)) > 0 Then
                Exit For
            End If
        Next
        start = i1
        lenght = i2 - i1
    End Sub

    ' ----------------------------------------------------------------------------------
    '  Extract from "text" start and length of the word pointed by the "selectionIndex"
    '  This version find shortest words - not including characters like  " , / () ...
    ' ----------------------------------------------------------------------------------
    Friend Sub FindWordPointers_ShortWords(ByVal text As String, _
                                            ByVal selectionIndex As Int32, _
                                            ByRef start As Int32, _
                                            ByRef lenght As Int32)
        Const separators As String = " "";:,/\|()'" + vbCr + vbLf + vbTab
        Dim i1, i2 As Int32
        For i1 = selectionIndex To 1 Step -1
            If InStr(separators, text(i1 - 1)) > 0 Then
                Exit For
            End If
        Next
        For i2 = selectionIndex To text.Length - 1
            If InStr(separators, text(i2)) > 0 Then
                Exit For
            End If
        Next
        start = i1
        lenght = i2 - i1
    End Sub


    ' ----------------------------------------------------------------------------------
    '  Extract numeric params from a string
    '  Valid only if fields are not more than MaxFieldNumber 
    ' ----------------------------------------------------------------------------------
    Friend Function ExtractNumericParams(ByVal s As String, ByVal MaxFieldsNumber As Int32) As Double()
        Dim n(-1) As Double
        Dim sa As String() = s.Split(" "c)
        Dim paramsCount As Int32 = 0
        For i As Int32 = 0 To sa.Length - 1
            If sa(i) <> "" Then
                If IsValidDouble(sa(i)) Then
                    ReDim Preserve n(n.Length)
                    n(n.Length - 1) = EvalDouble(sa(i))
                End If
                paramsCount += 1
            End If
        Next
        If paramsCount > MaxFieldsNumber Then ReDim n(-1)
        Return n
    End Function

    ' =======================================================================================
    '  TrayWnd
    ' =======================================================================================
    'Private Declare Function FindWindow Lib "user32" Alias "FindWindowA" (ByVal lpClassName As String, ByVal lpWindowName As String) As Int32
    'Private Declare Function ShowWindow Lib "user32" (ByVal hwnd As Int32, ByVal nCmdShow As Int32) As Int32
    'Private Const SW_HIDE As Int32 = 0
    'Private Const SW_SHOW As Int32 = 5

    'Friend Sub HideTrayWnd()
    '    Dim lngHandle As Int32
    '    lngHandle = FindWindow("Shell_TrayWnd", "")
    '    ShowWindow(lngHandle, SW_HIDE)
    'End Sub

    'Friend Sub ShowTrayWnd()
    '    Dim lngHandle As Int32
    '    lngHandle = FindWindow("Shell_TrayWnd", "")
    '    ShowWindow(lngHandle, SW_SHOW)
    'End Sub

    ' =============================================================================
    '   START PROCESS (IF NOT RUNNING) AND WAIT
    ' =============================================================================
    Friend Sub StartProcess_IfNotRunning_AndWait(ByVal filename As String)
        ' ------------------------------------------------------------------ Prepare path
        If Not IO.Path.IsPathRooted(filename) Then
            filename = Application.StartupPath + "\" + filename
        End If
        If Not IO.File.Exists(filename) Then Return
        ' ------------------------------------------------------------------ Test if already running
        If AppIsRunning(filename) Then Return
        ' ------------------------------------------------------------------ Init process
        Dim proc As Process
        Dim psi As New ProcessStartInfo
        psi.WorkingDirectory = IO.Path.GetDirectoryName(filename)
        psi.FileName = filename
        ' ------------------------------------------------------------------ protect for process errors
        Try
            ' -------------------------------------------------------------- Start process
            proc = Process.Start(psi)
            ' -------------------------------------------------------------- Wait process really started 
            For i As Int32 = 1 To 100
                Threading.Thread.Sleep(100)
                proc.Refresh()
                If proc.MainWindowTitle.Contains(".") Then Exit For
            Next
        Catch
        End Try
        Threading.Thread.Sleep(500)
    End Sub

    ' =============================================================================
    '   App Is Running  (testing the complete path of the running app)
    ' =============================================================================
    Friend Function AppIsRunning(ByVal FileName As String) As Boolean
        ' ------------------------------------------------------------------ Correct path for /../..
        FileName = IO.Path.GetFullPath(FileName)
        ' ------------------------------------------------------------------ Normalize filename
        FileName = FileName.Replace("/", "\")
        ' ------------------------------------------------------------------ Compare with all processes
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            Try
                If p(i).MainWindowTitle = "" Then Continue For
                If p(i).MainModule.FileName = FileName Then Return True
            Catch
            End Try
        Next
        Return False
    End Function

    ' ==============================================================================================================
    '   Close All ThereminoApps
    ' ==============================================================================================================
    Friend Sub CloseAllThereminoApps()
        Dim p() As System.Diagnostics.Process
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If Not p(i).ProcessName.StartsWith("Theremino_", StringComparison.OrdinalIgnoreCase) Then Continue For
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            'Debug.Print(p(i).ProcessName)
            '
            p(i).CloseMainWindow()
            p(i).Close()
        Next
        '
        p = System.Diagnostics.Process.GetProcesses()
        For i As Int32 = 0 To p.Length - 1
            If Not p(i).ProcessName.StartsWith("Theremino_", StringComparison.OrdinalIgnoreCase) Then Continue For
            If p(i).Id = System.Diagnostics.Process.GetCurrentProcess.Id Then Continue For
            '
            'Debug.Print(p(i).ProcessName)
            '
            If Not p(i).WaitForExit(1000) Then
                Try
                    p(i).Kill()
                Catch ex As Exception
                End Try
            End If
        Next
    End Sub

    ' ==============================================================
    '  REPLACE MULTIPLE SPACES and TABS and TRIM
    '  (3..5 uS for Strings with 32 chars)
    ' ============================================================== 
    Friend Function ReplaceMultipleSpacesAndTabAndTrim(ByVal s As String) As String
        s = s.Replace(vbTab, " ")
        While s.Contains("  ")
            s = s.Replace("  ", " ")
        End While
        Return s.Trim
    End Function

    'Friend Function ReplaceMultipleSpacesAndTrim_PreserveQuotedSpaces(ByVal s As String) As String
    '    ' ------------------------------------------------- replace inside quotes spaces with chr(255)
    '    Dim InsideQuotes As Boolean = False
    '    Dim s2 As String = ""
    '    For i As Int32 = 0 To s.Length - 1
    '        If s(i) = """" Then InsideQuotes = Not InsideQuotes
    '        If InsideQuotes Then
    '            If s(i) = " " Then
    '                s2 += Chr(160)   'NBSP
    '            Else
    '                s2 += s(i)
    '            End If
    '        Else
    '            s2 += s(i)
    '        End If
    '    Next
    '    ' ------------------------------------------------- remove all double spaces
    '    While s2.Contains("  ")
    '        s2 = s2.Replace("  ", " ")
    '    End While
    '    ' ------------------------------------------------- replace inside quotes chr(255) with spaces
    '    's2 = s2.Replace(Chr(160), " ")
    '    ' ------------------------------------------------- trim
    '    Return s2.Trim
    'End Function


    ' =============================================================================
    '   GetRightPartAfterThisWord
    ' =============================================================================
    Friend Function GetRightPartAfterThisLowerCaseWord(ByVal str As String, ByVal word As String) As String
        Dim i As Int32 = str.ToLower.IndexOf(word) + word.Length
        If i > 1 And i < str.Length Then
            Return str.Remove(0, i)
        Else
            Return ""
        End If
    End Function



    ' ======================================================================================================
    '  ASYNC KEY STATE
    ' ======================================================================================================

    Private Declare Function GetAsyncKeyState Lib "user32" (ByVal vKey As Int32) As Int16

    Private Const PRESSED_NOW As Int32 = &H8000
    Private Const PRESSED_AFTER_PREVIOUS_CALL As Int32 = &H1

    Friend Function Key(ByVal k As Int32) As Boolean
        Key = (GetAsyncKeyState(k) And PRESSED_NOW) <> 0
    End Function

    Friend Function KeyPrevious(ByVal k As Int32) As Boolean
        KeyPrevious = (GetAsyncKeyState(k) And PRESSED_AFTER_PREVIOUS_CALL) <> 0
    End Function

    Friend Sub WaitMouseOff()
        While Key(Keys.LButton) Or Key(Keys.MButton) Or Key(Keys.RButton)
            System.Threading.Thread.Sleep(1)
        End While
    End Sub

    Friend Sub WaitKeyOff(ByVal WaitKey As Int32)
        Do
            System.Threading.Thread.Sleep(1)
        Loop Until Not Key(WaitKey)
    End Sub


    ' ==============================================================
    '  RICH TEXT BOX - SUPPRESS SOUNDS (call from KeyDown)
    ' ==============================================================
    Friend Sub RichTextBox_SuppressSounds(ByVal sender As Object, ByRef e As KeyEventArgs)
        Dim rtb As RichTextBox = CType(sender, RichTextBox)
        If rtb.GetLineFromCharIndex(rtb.SelectionStart) = 0 AndAlso _
                                            (e.KeyData = Keys.Up Or e.KeyData = Keys.PageUp) _
        OrElse rtb.GetLineFromCharIndex(rtb.SelectionStart) = rtb.GetLineFromCharIndex(rtb.TextLength) AndAlso _
                                            (e.KeyData = Keys.Down Or e.KeyData = Keys.PageDown) _
        OrElse rtb.SelectionStart = rtb.TextLength AndAlso e.KeyData = Keys.Right _
        OrElse rtb.SelectionStart = 0 AndAlso _
                (e.KeyData = Keys.Left Or (e.KeyData = Keys.Back And rtb.SelectionLength = 0)) Then
            e.Handled = True
        End If
    End Sub


    ' ======================================================================================================
    '  DEBUG FUNCTIONS (Debug window)
    ' ======================================================================================================
    Private DebugStopWatch As Stopwatch = New Stopwatch
    Friend Sub Debug_InitTime()
        DebugStopWatch.Reset()
        DebugStopWatch.Start()
    End Sub
    Friend Sub Debug_Print(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s)
    End Sub
    Friend Sub Debug_PrintMicro(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000000", GCI).PadLeft(8) + " " + s)
    End Sub
    Friend Sub Debug_PrintDiffTime(ByVal s As String)
        Debug.Print(DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s)
        Debug_InitTime()
    End Sub

    ' ======================================================================================================
    '  DEBUG FUNCTIONS (Form1 text)
    ' ======================================================================================================
    'Private DebugStopWatch As Stopwatch = New Stopwatch
    'Friend Sub Debug_InitTime()
    '    DebugStopWatch.Reset()
    '    DebugStopWatch.Start()
    'End Sub
    'Friend Sub Debug_Print(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s
    'End Sub
    'Friend Sub Debug_PrintMicro(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000000", GCI).PadLeft(8) + " " + s
    'End Sub
    'Friend Sub Debug_PrintDiffTime(ByVal s As String)
    '    Form1.Text = DebugStopWatch.Elapsed.TotalSeconds.ToString("0.000", GCI).PadLeft(8) + " " + s
    '    Debug_InitTime()
    'End Sub


    ' ======================================================================================================
    '  OVERWRITE MODE CARET
    ' ======================================================================================================
    Private Declare Function CreateCaret Lib "user32" (ByVal hWnd As IntPtr, _
                                                      ByVal hBitmap As IntPtr, _
                                                      ByVal nWidth As Int32, _
                                                      ByVal nHeight As Int32) As Int32

    Private Declare Function ShowCaret Lib "user32" (ByVal hWnd As IntPtr) As Int32

    Private OverwriteMode As Boolean

    Friend Sub OverwriteMode_Toggle(ByRef rtb As Theremino_Automation.SyntaxRichTextBox)
        OverwriteMode = Not OverwriteMode
        If Not OverwriteMode Then
            CreateCaret(rtb.Handle, IntPtr.Zero, 1, CInt((rtb.Font.Height - 1) * rtb.ZoomFactor))
            ShowCaret(rtb.Handle)
        End If
    End Sub

    Friend Sub OverwriteMode_ShowCaret(ByRef rtb As Theremino_Automation.SyntaxRichTextBox)
        If OverwriteMode Then
            CreateCaret(rtb.Handle, IntPtr.Zero, CInt(12 * rtb.ZoomFactor), CInt((rtb.Font.Height - 1) * rtb.ZoomFactor))
            ShowCaret(rtb.Handle)
        End If
    End Sub


    ' ======================================================================================================
    '  STRING SPACED AROUND OPERATORS 
    ' ======================================================================================================
    Friend Function StringSpacedAroundOperators(ByVal s As String) As String
        s = SpacesAroundOperators(s)
        s = ReplaceMultipleSpacesAndTabAndTrim(s)
        Return " " + s + " "
    End Function

    Private SubtractionChars As String = ChrW(&H2D) + " " + _
                                         ChrW(&H2010) + " " + _
                                         ChrW(&H2212) + " " + _
                                         ChrW(&H2013) + " " + _
                                         ChrW(&H2014)

    Dim Operators As String = SubtractionChars + " ( ) * / + = ^ ,"

    Dim OperatorsArray() As String = Operators.Replace("  ", " ").Split(" "c)

    Private Function SpacesAroundOperators(ByVal s As String) As String
        If s = "" Then Return s
        For Each Str As String In OperatorsArray
            If Str <> "" Then
                s = s.Replace(Str, " " + Str + " ")
            End If
        Next
        Return s
    End Function

End Module
