﻿
Module Module_Variables

    ' =======================================================================================
    '  VARIABLES - SHORT VARS and USER DEFINED VARIABLES
    ' =======================================================================================
    Friend Const UserVariablesMinLength As Int32 = 2
    Friend NumericShortVars(10) As Double
    Friend StringShortVars() As String = {"", "", "", "", "", "", "", "", "", "", ""}

    Structure NumericUserVariable
        Dim Name As String
        Dim Value As Double
        Dim Nline As Int32
    End Structure
    Structure StringUserVariable
        Dim Name As String
        Dim Value As String
        Dim Nline As Int32
    End Structure

    Private NumericUserVariableArray(-1) As NumericUserVariable
    Private StringUserVariableArray(-1) As StringUserVariable
    Private AllVariableNames(-1) As String

    Friend Function GetAllVariableNames() As String()
        Return AllVariableNames
    End Function

    ' =======================================================================================
    '  VARIABLES - REINIT USER DEFINED VARIABLES
    ' =======================================================================================
    Friend Sub ReinitUserVariables()
        ' ----------------------------------------------------- speed test 3 mS with 2000 lines
        'Dim sw As Stopwatch = New Stopwatch
        'sw.Start()
        ' ----------------------------------------------------- 
        RTBLines = Form1.RTB.Lines
        ' ----------------------------------------------------- empty the arrays
        ReDim NumericUserVariableArray(-1)
        ReDim StringUserVariableArray(-1)
        ReDim AllVariableNames(-1)
        ' ----------------------------------------------------- scan all the lines
        For i As Int32 = 0 To RTBLines.Length - 1
            Dim line As String = RTBLines(i)
            If line.Trim.ToLower.StartsWith("variable") Then
                Dim sa() As String = ReplaceMultipleSpacesAndTabAndTrim(line).Split(" "c)
                If sa.Length >= 3 Then
                    If TestUserVariableName(sa(2)) = "OK" Then
                        Select Case sa(1).ToLower
                            Case "numeric"
                                ReDim Preserve NumericUserVariableArray(NumericUserVariableArray.Length)
                                NumericUserVariableArray(NumericUserVariableArray.Length - 1).Name = sa(2)
                                'NumericUserVariableArray(NumericUserVariableArray.Length - 1).Value = 0
                                NumericUserVariableArray(NumericUserVariableArray.Length - 1).Nline = i
                            Case "string"
                                ReDim Preserve StringUserVariableArray(StringUserVariableArray.Length)
                                StringUserVariableArray(StringUserVariableArray.Length - 1).Name = sa(2)
                                StringUserVariableArray(StringUserVariableArray.Length - 1).Value = ""
                                StringUserVariableArray(StringUserVariableArray.Length - 1).Nline = i
                        End Select
                        ReDim Preserve AllVariableNames(AllVariableNames.Length)
                        AllVariableNames(AllVariableNames.Length - 1) = sa(2)
                    End If
                End If
            End If
        Next
        'Form1.Text = VariableNumericArray.Length.ToString + " " + VariableStringArray.Length.ToString
        'Form1.Text = sw.Elapsed.TotalMilliseconds.ToString
    End Sub

    Private ReservedKeywords As String = " slot sin cos asin acos tan atan atan2" + _
                                         " pow sqrt abs sign int round mid len trim" + _
                                         " padleft padright format ucase lcase wcase " + _
                                         " crlf cr lf math_pi math_e" + _
                                         " nbsp tab chars date now today elapsedtime rnd" + _
                                         " disabled enabled input true false" + _
                                         " seconds sleep space esc reset" + _
                                         " text hide home left right up down pageup pagedown" + _
                                         " endselect sleep padleft padright replace" + _
                                         " beep button gosub goto if else endif" + _
                                         " key label load print return save select case" + _
                                         " caseelse endSelect slot speed stop end" + _
                                         " variable wait window slots vars hide closeapps" + _
                                         " com_status com_received com_portnames" + _
                                         " getseparatedstringcount getseparatedstring" + _
                                         " controls opentextbox closetextbox cleartextbox"

    Friend Function TestUserVariableName(ByVal name As String) As String
        ' --------------------------------- Identifiers may only begin with a letter or underscore
        Dim c As Char = name(0)
        If (c < "a" Or c > "z") And (c < "A" Or c > "Z") And c <> "_" Then
            Return "A variable name can not start with numbers or signs"
        End If
        ' --------------------------------- Identifiers may only contain letters, numbers and underscores
        For i As Int32 = 1 To name.Length - 1
            c = name(i)
            If (c < "a" Or c > "z") And (c < "A" Or c > "Z") And (c < "0" Or c > "9") And c <> "_" Then
                Return "A variable name can only contain letters, numbers and underscores"
            End If
        Next
        ' --------------------------------- test if it is a reserved word
        If ReservedKeywords.Contains(" " + name.ToLower + " ") Then
            Return "This variable name is a reserved word"
        End If
        ' --------------------------------- test if already exists
        If IsSomeVariable(name) Then
            Return "This variable name already exists"
        End If
        ' --------------------------------- test if too short
        If name.Length < UserVariablesMinLength Then
            Return "This variable name is too short"
        End If
        ' ---------------------------------
        Return "OK"
    End Function

    ' =======================================================================================
    '  TEST VARIABLES
    ' =======================================================================================
    Friend Function IsSomeVariable(ByVal name As String) As Boolean
        If name Is Nothing Then Return False
        If IsNumericUserVariable(name) Then Return True
        If IsStringUserVariable(name) Then Return True
        If IsNumericShortVariable(name) Then Return True
        If IsStringShortVariable(name) Then Return True
        Return False
    End Function
    Friend Function IsNumericVariable(ByVal name As String) As Boolean
        If name Is Nothing Then Return False
        If IsNumericUserVariable(name) Then Return True
        If IsNumericShortVariable(name) Then Return True
        Return False
    End Function
    Friend Function IsStringVariable(ByVal name As String) As Boolean
        If name Is Nothing Then Return False
        If IsStringUserVariable(name) Then Return True
        If IsStringShortVariable(name) Then Return True
        Return False
    End Function

    Friend Function IsNumericUserVariable(ByVal name As String) As Boolean
        For i As Int32 = 0 To NumericUserVariableArray.Length - 1
            If name.ToLower = NumericUserVariableArray(i).Name.ToLower Then Return True
        Next
        Return False
    End Function
    Friend Function IsStringUserVariable(ByVal name As String) As Boolean
        For i As Int32 = 0 To StringUserVariableArray.Length - 1
            If name.ToLower = StringUserVariableArray(i).Name.ToLower Then Return True
        Next
        Return False
    End Function

    Friend Function IsNumericUserVariable(ByVal name As String, ByVal line As Int32) As Boolean
        For i As Int32 = 0 To NumericUserVariableArray.Length - 1
            If name.ToLower = NumericUserVariableArray(i).Name.ToLower And line = NumericUserVariableArray(i).Nline Then Return True
        Next
        Return False
    End Function
    Friend Function IsStringUserVariable(ByVal name As String, ByVal line As Int32) As Boolean
        For i As Int32 = 0 To StringUserVariableArray.Length - 1
            If name.ToLower = StringUserVariableArray(i).Name.ToLower And line = StringUserVariableArray(i).Nline Then Return True
        Next
        Return False
    End Function

    Friend Function IsNumericShortVariable(ByVal name As String) As Boolean
        Select Case name.ToLower
            Case "v1", "v2", "v3", "v4", "v5", "v6", "v7", "v8", "v9"
                Return True
        End Select
        Return False
    End Function
    Friend Function IsStringShortVariable(ByVal name As String) As Boolean
        Select Case name.ToLower
            Case "s1", "s2", "s3", "s4", "s5", "s6", "s7", "s8", "s9"
                Return True
        End Select
        Return False
    End Function

    Friend Function TheLineContainsSomeUserVariable(ByVal l As String) As Boolean
        l = StringSpacedAroundOperators(l)
        Dim sa() As String = l.Split(" "c)
        For Each s As String In sa
            If IsNumericUserVariable(s) Then Return True
            If IsStringUserVariable(s) Then Return True
        Next
        Return False
    End Function


    ' =======================================================================================
    '  SET and GET VARIABLES
    ' =======================================================================================
    Friend Sub SetVariable(ByVal name As String, ByVal value As String)
        If IsNumericUserVariable(name) Then SetNumericUserVariable(name, Val(value)) : Return
        If IsStringUserVariable(name) Then SetStringUserVariable(name, value) : Return
        If IsNumericShortVariable(name) Then SetNumericShortVar(name, Val(value)) : Return
        If IsStringShortVariable(name) Then SetStringShortVar(name, value) : Return
    End Sub

    Friend Sub SetNumericVariable(ByVal name As String, ByVal value As Double)
        If IsNumericUserVariable(name) Then SetNumericUserVariable(name, value) : Return
        If IsNumericShortVariable(name) Then SetNumericShortVar(name, value) : Return
    End Sub

    Friend Function GetNumericVariable(ByVal name As String) As Double
        If IsNumericUserVariable(name) Then Return GetNumericUserVariable(name)
        If IsNumericShortVariable(name) Then Return GetNumericShortVariable(name)
    End Function

    Friend Sub GetVariable(ByVal name As String, ByRef value As Object)
        If IsNumericUserVariable(name) Then
            value = GetNumericUserVariable(name)
        ElseIf IsStringUserVariable(name) Then
            value = GetStringUserVariable(name)
        ElseIf IsNumericShortVariable(name) Then
            value = GetNumericShortVariable(name)
        ElseIf IsStringShortVariable(name) Then
            value = GetStringShortVariable(name)
        End If
    End Sub

    Friend Sub SetNumericShortVar(ByVal name As String, ByVal value As Double)
        Select Case name.ToLower
            Case "v1" : NumericShortVars(1) = value
            Case "v2" : NumericShortVars(2) = value
            Case "v3" : NumericShortVars(3) = value
            Case "v4" : NumericShortVars(4) = value
            Case "v5" : NumericShortVars(5) = value
            Case "v6" : NumericShortVars(6) = value
            Case "v7" : NumericShortVars(7) = value
            Case "v8" : NumericShortVars(8) = value
            Case "v9" : NumericShortVars(9) = value
        End Select
    End Sub

    Friend Sub SetStringShortVar(ByVal name As String, ByVal value As String)
        Select Case name.ToLower
            Case "s1" : StringShortVars(1) = value
            Case "s2" : StringShortVars(2) = value
            Case "s3" : StringShortVars(3) = value
            Case "s4" : StringShortVars(4) = value
            Case "s5" : StringShortVars(5) = value
            Case "s6" : StringShortVars(6) = value
            Case "s7" : StringShortVars(7) = value
            Case "s8" : StringShortVars(8) = value
            Case "s9" : StringShortVars(9) = value
        End Select
    End Sub

    Friend Function GetNumericShortVariable(ByVal name As String) As Double
        Select Case name.ToLower
            Case "v1" : Return NumericShortVars(1)
            Case "v2" : Return NumericShortVars(2)
            Case "v3" : Return NumericShortVars(3)
            Case "v4" : Return NumericShortVars(4)
            Case "v5" : Return NumericShortVars(5)
            Case "v6" : Return NumericShortVars(6)
            Case "v7" : Return NumericShortVars(7)
            Case "v8" : Return NumericShortVars(8)
            Case "v9" : Return NumericShortVars(9)
        End Select
    End Function

    Friend Function GetStringShortVariable(ByVal name As String) As String
        Select Case name.ToLower
            Case "s1" : Return StringShortVars(1)
            Case "s2" : Return StringShortVars(2)
            Case "s3" : Return StringShortVars(3)
            Case "s4" : Return StringShortVars(4)
            Case "s5" : Return StringShortVars(5)
            Case "s6" : Return StringShortVars(6)
            Case "s7" : Return StringShortVars(7)
            Case "s8" : Return StringShortVars(8)
            Case "s9" : Return StringShortVars(9)
        End Select
        Return ""
    End Function

    ' =======================================================================================
    '  SET and GET - USER DEFINED VARIABLES
    ' =======================================================================================
    Friend Sub SetNumericUserVariable(ByVal name As String, ByVal value As Double)
        For i As Int32 = 0 To NumericUserVariableArray.Length - 1
            If name.ToLower = NumericUserVariableArray(i).Name.ToLower Then
                NumericUserVariableArray(i).Value = value
            End If
        Next
    End Sub
    Friend Sub SetStringUserVariable(ByVal name As String, ByVal value As String)
        For i As Int32 = 0 To StringUserVariableArray.Length - 1
            If name.ToLower = StringUserVariableArray(i).Name.ToLower Then
                StringUserVariableArray(i).Value = value
            End If
        Next
    End Sub

    Friend Function GetNumericUserVariable(ByVal name As String) As Double
        For i As Int32 = 0 To NumericUserVariableArray.Length - 1
            If name.ToLower = NumericUserVariableArray(i).Name.ToLower Then
                Return NumericUserVariableArray(i).Value
            End If
        Next
        Return 0
    End Function
    Friend Function GetStringUserVariable(ByVal name As String) As String
        For i As Int32 = 0 To StringUserVariableArray.Length - 1
            If name.ToLower = StringUserVariableArray(i).Name.ToLower Then
                Return StringUserVariableArray(i).Value
            End If
        Next
        Return ""
    End Function

    Friend Function GetVariablesStartingWith(ByVal initial As String) As String
        Dim s As String = ""
        For Each v As NumericUserVariable In NumericUserVariableArray
            If v.Name.ToLower.StartsWith(initial) Then
                s += v.Name + " / "
            End If
        Next
        For Each v As StringUserVariable In StringUserVariableArray
            If v.Name.ToLower.StartsWith(initial) Then
                s += v.Name + " / "
            End If
        Next
        If initial = "v" Then
            s += "v1..v9 / "
        End If
        If initial = "s" Then
            s += "s1..s9 / "
        End If
        Return s
    End Function

End Module
